/**
 * 企业融资相关逻辑
 * 作者: 李欣鸣
 * 包括新融资信息的增删改查
 * 
 */

import { FinancingParamConfig } from "../../config/checkParamConfig";
import { ERRORENUM } from "../../config/errorEnum";
import { EnterpriseFinancingInfoConfig, EnterpriseInfoConfig } from "../../config/ojbectResultKeyConfig";
import { createFinancing, deleteEnterpriseFinancing, findAddByName, findEnterpriseFinancingByUscc, findFinancingByName, findFinancingCount, findFinancingInfoByTaskIdAndSucc, findFinancingList, groupFindFinancing } from "../../data/enterprise/financing";
import { FinancingListConfig } from "../../config/ojbectResultKeyConfig";
import { BizError } from "../../util/bizError";
import { checkChange, checkParamater, extractData, getTaskId } from "../../util/tools";


/**
 * 创建企业融资信息
 * @param uscc 孵化器统一信用代码
 * @param param 表单配置
 * @returns {isSuccess:true/false}
 */
export async function createFinancingInfo(uscc, param) {
    const TaskId = getTaskId(uscc);

    checkParamater("录入企业融资信息", FinancingParamConfig, param);

    let dataBaseInfo = await findFinancingInfoByTaskIdAndSucc(TaskId, param.uscc);
    
    if ( dataBaseInfo&& dataBaseInfo.uscc)  throw new BizError(ERRORENUM.该企业当月数据已存在, `${param.uscc}的融资信息已经存在`);

    await createFinancing(uscc, TaskId, param);

    return {isSuccess:true};
}

/**
 * 更新融资企业信息
 * @param uscc 孵化器统一信用代码
 * @param param 表单配置
 * @returns {isSuccess:true/false}
 */
export async function updateFinancingInfo(uscc, param) {
    const TaskId = getTaskId(uscc);

    checkParamater("更新融资企业信息", FinancingParamConfig, param);
    let dataBaseInfo = await findFinancingInfoByTaskIdAndSucc(TaskId, param.uscc);
    let changeList = checkChange(param, dataBaseInfo);

    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${TaskId}的任务`);
    if ( !changeList.length ) throw new BizError(ERRORENUM.数据无更新, `${param.uscc}数据无更新`);

    changeList.forEach(key => {
        dataBaseInfo[key] = param[key];
    });

    await dataBaseInfo.save();

    return {isSuccess:true};
}


/**
 * 根据企业id查询企业
 * @param uscc 企业统一信用代码
 */
export async function getEnterpriseFinancingByUscc(fuHuaQiUscc:string, uscc:string) {
    const TaskId = getTaskId(fuHuaQiUscc);
    let dataBaseInfo = await findEnterpriseFinancingByUscc(uscc, TaskId);
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${uscc}这个企业`);

    let data = extractData(EnterpriseFinancingInfoConfig, dataBaseInfo);

    return {data};
}


/**
 * 根据企业id删除草稿企业
 * @param uscc 企业统一信用代码
 */
export async function deleteEnterpriseFinancingByUscc(fuHuaQiUscc:string, uscc:string) {
    const TaskId = getTaskId(fuHuaQiUscc);
    let dataBaseInfo = await findEnterpriseFinancingByUscc(uscc, TaskId);
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${uscc}这个企业`);

    if (dataBaseInfo.draftLock) throw new BizError(ERRORENUM.已入库的企业不能删除, uscc);

    await deleteEnterpriseFinancing(uscc, TaskId);

    return {isSuccess:true};
}
/**
 * 融资企业名称模糊查询
 * @param name 融资企业名称
 */
export async function getFinancingByName(name:string) {
    let financingInfo = await findFinancingByName(name);
    let dataList = [];
    financingInfo.forEach( info => {
        dataList.push(info.name);
    });
    return dataList;
}

/**
 * 根据融资企业名称获取注册地址、经营地址
 * @param name 
 */
export async function getAddByName(name:string) {
    let financingInfo = await findAddByName(name);
    let  dataList = [];
    let keyList  = ["logonAdd", "operatingAdd"];
    keyList.forEach( key => {
        dataList[key] = financingInfo[key];
    })
    return dataList;
}

/**
 * 融资企业信息列表
 * @param month 数据月份 目前数据库没有该字段
 * @param fuHuaQiUscc 孵化器统一信用代码 实际参数是所属孵化器 
 * @param industry 行业领域 根据孵化器信用代码查找 fuhuaqi表的领域
 * @param fuHuaQiInvestment 孵化器是否参与投资
 * @param page 页数 
 * @returns 
 */
export async function financingList(fuHuaQiUscc:string, fuHuaQiInvestment:boolean, page:number) {
    let selectParam:any = {};
    if (fuHuaQiUscc) selectParam.fuHuaQiUscc = fuHuaQiUscc;
    if (fuHuaQiInvestment) selectParam.fuHuaQiInvestment = fuHuaQiInvestment;

    let enterpriseMap = await groupFindFinancing();
    let dataBaseList = await findFinancingList(selectParam, (page-1)*10 );
    let count = await findFinancingCount(selectParam);
    let dataList = [];
    dataBaseList.forEach( info => {
        let changeData = extractData(FinancingListConfig, info);
        const Count = enterpriseMap[info.uscc] || 0;
        dataList.push(changeData);
    });

    return {count, dataList};
}